classdef EltMesh < handle % Elementary mesh
  
  properties %(SetAccess = protected)
    d =0; % space dimension  
    m=0;  % m-faces
    order=0;
    stype='simplex';
    type=0; % mesh type : O -> d-simplicial, 1 -> d-orthotope
    nq=0;
    nme=0;
    q  = [];
    me = [];
    toGlobal=[];
    vidx=[]; % vertices index
    label='';
    %Tag='';
    color=[];
  end
  
  methods
    function self = EltMesh(m,q,me,varargin)
      [d,nq]=size(q);
      assert( m <= d);
      p = inputParser;
      p.addParamValue('type', 'simplex', @(x) ismember(x,{'simplex','orthotope'}) );
      p.addParamValue('color', [0 0 1] );
      p.addParamValue('label', '', @ischar );
      p.addParamValue('order', 1, @isscalar );
      p.addParamValue('toGlobal', 1:nq );
      p.parse(varargin{:});
      R=p.Results;
      self.stype=R.type;
      self.type=strcmp(R.type,'orthotope'); % 1: orthotope, 0: simplex
      assert( size(me,1)==  (self.type*(R.order+1)^m + (1-self.type)*nchoosek(m+R.order,R.order)), ...
              'Trouble with me size!')
      
      self.d=d;
      self.m=m;
      self.order=R.order;
      self.q=q;
      self.nq=nq;
      self.me=me;
      self.nme=size(me,2);
      
      self.toGlobal=R.toGlobal;
      self.label=R.label;
      self.color=R.color;
      self.vidx=fc_hypermesh.VerticesIdxRef(m,R.order,'type',R.type);
      
    end % EltMesh
    
    function disp(self)
    n=length(self);
      if n==1
        fprintf('EltMesh object \n');
        fc_hypermesh.EltMesh_fun.info_one(self)
      else
        fprintf('Array of %d EltMesh objects \n',n);
        for i=1:n
          fprintf('  Element %d\n',i)
          fc_hypermesh.EltMesh_fun.info_one(self(i))
        end
      end
    end % disp method  
    
    function Ba=barycenters(self)
      for i=self.vidx
        Ba=Ba+q(:,me(i,:));
      end
      Ba=Ba/length(self.vidx);
    end % barycenters method

    function varargout=plotmesh(self,varargin)
      h=fc_hypermesh.EltMesh_fun.plotmesh(self, varargin{:});
      if nargout==1,varargout{1}=h;end
    end % plotmesh method
    
    function plotnodes(self,varargin)
      fc_hypermesh.EltMesh_fun.plotnodes(self, varargin{:});
    end % plotnodes method
    
    function plotnodesNumber(self,varargin)
      fc_hypermesh.EltMesh_fun.plotnodesNumber(self, varargin{:});
    end % plotnodesNumber method
    
    function ploteltsNumber(self,varargin)
      fc_hypermesh.EltMesh_fun.ploteltsNumber(self, varargin{:});
    end % plotEltsNumber method
    
    function set(self,varargin)
      % Create for Octave 4.2.0:
      %   if self.label='toto' is used then a new empty object is created...
      %   To verify ...
      p = inputParser;
      p.addParamValue('color', [] );
      p.addParamValue('label', [] );
      p.addParamValue('mapping', [] );
      p.parse(varargin{:});
      R=p.Results;
      if ~isempty(R.color)
        self.color=R.color;
      end
      if ~isempty(R.label)
        self.label=R.label;
      end
      if ~isempty(R.mapping)
        self.q=R.mapping(self.q);
      end
    end
    
  end % methods
end %classdef


